@@ -6,6 +6,8 @@ from sqlfluff.core.parser.segments.base import (
     UnparsableSegment,
     BracketedSegment,
     IdentitySet,
+    FixPatch,
+    SourceFix,
 )
 from sqlfluff.core.parser.segments.generator import SegmentGenerator
 from sqlfluff.core.parser.segments.raw import (
@@ -47,4 +49,6 @@ __all__ = (
     "Dedent",
     "TemplateSegment",
     "IdentitySet",
+    "FixPatch",
+    "SourceFix",
 )
