@@ -112,6 +112,7 @@ BEGIN
 BERNOULLI
 BINARY
 BINDING
+BREAK
 CACHE
 CALL
 CASCADE
@@ -124,6 +125,7 @@ COLUMNS
 COMMENT
 COMMIT
 CONCURRENTLY
+CONTINUE
 CONNECT
 CONNECTION
 CONSTRAINT
@@ -142,6 +144,7 @@ DO
 DOMAIN
 DOUBLE
 DROP
+ELSEIF
 EXECUTE
 EXECUTION
 EXPLAIN
@@ -167,10 +170,13 @@ INDEX
 INOUT
 INSERT
 INTEGRATION
+ITERATE
 KEY
 LANGUAGE
 LARGE
 LAST
+LEAVE
+LOOP
 MANAGE
 MASKING
 MATCHED
@@ -218,6 +224,7 @@ READ
 REFERENCE_USAGE
 REFERENCES
 RENAME
+REPEAT
 REPEATABLE
 REPLACE
 RESOURCE
@@ -260,6 +267,7 @@ TRUNCATE
 TYPE
 UNIQUE
 UNSIGNED
+UNTIL
 UPDATE
 USAGE
 USE
@@ -271,6 +279,7 @@ VARYING
 VERSION
 VIEW
 WAREHOUSE
+WHILE
 WITHOUT
 WORK
 WRAPPER
