@@ -204,6 +204,30 @@ hive_dialect.replace(
         ],
         before=Sequence("ORDER", "BY"),
     ),
+    GroupByClauseTerminatorGrammar=OneOf(
+        Sequence(
+            OneOf("ORDER", "CLUSTER", "DISTRIBUTE", "SORT"),
+            "BY",
+        ),
+        "LIMIT",
+        "HAVING",
+        "QUALIFY",
+        "WINDOW",
+    ),
+    HavingClauseTerminatorGrammar=OneOf(
+        Sequence(
+            OneOf(
+                "ORDER",
+                "CLUSTER",
+                "DISTRIBUTE",
+                "SORT",
+            ),
+            "BY",
+        ),
+        "LIMIT",
+        "QUALIFY",
+        "WINDOW",
+    ),
 )
 
 
@@ -838,36 +862,6 @@ class SelectClauseSegment(ansi.SelectClauseSegment):
     parse_grammar = ansi.SelectClauseSegment.parse_grammar.copy()
 
 
-class GroupByClauseSegment(ansi.GroupByClauseSegment):
-    """Overriding GroupByClauseSegment to allow for additional segment parsing."""
-
-    match_grammar = ansi.GroupByClauseSegment.match_grammar.copy()
-    match_grammar.terminator = match_grammar.terminator.copy(  # type: ignore
-        insert=[
-            Sequence("CLUSTER", "BY"),
-            Sequence("DISTRIBUTE", "BY"),
-            Sequence("SORT", "BY"),
-        ],
-        before=Ref.keyword("LIMIT"),
-    )
-    parse_grammar = ansi.GroupByClauseSegment.parse_grammar
-
-
-class HavingClauseSegment(ansi.HavingClauseSegment):
-    """Overriding HavingClauseSegment to allow for additional segment parsing."""
-
-    match_grammar = ansi.HavingClauseSegment.match_grammar.copy()
-    match_grammar.terminator = match_grammar.terminator.copy(  # type: ignore
-        insert=[
-            Sequence("CLUSTER", "BY"),
-            Sequence("DISTRIBUTE", "BY"),
-            Sequence("SORT", "BY"),
-        ],
-        before=Ref.keyword("LIMIT"),
-    )
-    parse_grammar = ansi.HavingClauseSegment.parse_grammar
-
-
 class SetExpressionSegment(ansi.SetExpressionSegment):
     """Overriding SetExpressionSegment to allow for additional segment parsing."""
 
