@@ -95,6 +95,7 @@ mysql_dialect.replace(
     LiteralGrammar=ansi_dialect.get_grammar("LiteralGrammar").copy(
         insert=[
             Ref("DoubleQuotedLiteralSegment"),
+            Ref("SystemVariableSegment"),
         ]
     ),
     FromClauseTerminatorGrammar=ansi_dialect.get_grammar(
@@ -208,6 +209,12 @@ mysql_dialect.add(
         type="literal",
         trim_chars=("@",),
     ),
+    SystemVariableSegment=RegexParser(
+        r"@@(session|global)\.[A-Za-z0-9_]+",
+        CodeSegment,
+        name="system_variable",
+        type="system_variable",
+    ),
 )
 
 
@@ -684,13 +691,13 @@ mysql_dialect.add(
         Ref("DatatypeSegment"),
     ),
     LocalVariableNameSegment=RegexParser(
-        r"`?[a-zA-Z0-9_]*`?",
+        r"`?[a-zA-Z0-9_$]*`?",
         CodeSegment,
         name="declared_variable",
         type="variable",
     ),
     SessionVariableNameSegment=RegexParser(
-        r"[@][a-zA-Z0-9_]*",
+        r"[@][a-zA-Z0-9_$]*",
         CodeSegment,
         name="declared_variable",
         type="variable",
@@ -710,7 +717,7 @@ mysql_dialect.insert_lexer_matchers(
     [
         RegexLexer(
             "at_sign",
-            r"[@][a-zA-Z0-9_]*",
+            r"@@?[a-zA-Z0-9_$]*(\.[a-zA-Z0-9_$]+)?",
             CodeSegment,
         ),
     ],
@@ -865,6 +872,7 @@ class StatementSegment(ansi.StatementSegment):
             Ref("UpsertClauseListSegment"),
             Ref("InsertRowAliasSegment"),
             Ref("FlushStatementSegment"),
+            Ref("LoadDataSegment"),
         ],
     )
 
@@ -1064,7 +1072,6 @@ class ProcedureParameterListGrammar(BaseSegment):
     match_grammar = Bracketed(
         Delimited(
             Ref("ProcedureParameterGrammar"),
-            delimiter=Ref("CommaSegment"),
             optional=True,
         ),
     )
@@ -2068,3 +2075,61 @@ class FlushStatementSegment(BaseSegment):
             ),
         ),
     )
+
+
+class LoadDataSegment(BaseSegment):
+    """A `LOAD DATA` statement.
+
+    As per https://dev.mysql.com/doc/refman/8.0/en/load-data.html
+    """
+
+    type = "load_data_statement"
+
+    match_grammar = Sequence(
+        "LOAD",
+        "DATA",
+        OneOf("LOW_PRIORITY", "CONCURRENT", optional=True),
+        Sequence("LOCAL", optional=True),
+        "INFILE",
+        Ref("QuotedLiteralSegment"),
+        OneOf("REPLACE", "IGNORE", optional=True),
+        "INTO",
+        "TABLE",
+        Ref("TableReferenceSegment"),
+        Sequence("PARTITION", Ref("SelectPartitionClauseSegment"), optional=True),
+        Sequence("CHARACTER", "SET", Ref("NakedIdentifierSegment"), optional=True),
+        Sequence(
+            OneOf("FIELDS", "COLUMNS"),
+            Sequence("TERMINATED", "BY", Ref("QuotedLiteralSegment"), optional=True),
+            Sequence(
+                Sequence("OPTIONALLY", optional=True),
+                "ENCLOSED",
+                "BY",
+                Ref("QuotedLiteralSegment"),
+                optional=True,
+            ),
+            Sequence("ESCAPED", "BY", Ref("QuotedLiteralSegment"), optional=True),
+            optional=True,
+        ),
+        Sequence(
+            "LINES",
+            Sequence("STARTING", "BY", Ref("QuotedLiteralSegment"), optional=True),
+            Sequence("TERMINATED", "BY", Ref("QuotedLiteralSegment"), optional=True),
+            optional=True,
+        ),
+        Sequence(
+            "IGNORE",
+            Ref("NumericLiteralSegment"),
+            OneOf("LINES", "ROWS"),
+            optional=True,
+        ),
+        Sequence(
+            Bracketed(Delimited(Ref("ColumnReferenceSegment"))),
+            optional=True,
+        ),
+        Sequence(
+            "SET",
+            Ref("Expression_B_Grammar"),
+            optional=True,
+        ),
+    )
