@@ -3,27 +3,31 @@
 This is based on postgres dialect, since it was initially based off of Postgres 8.
 We should monitor in future and see if it should be rebased off of ANSI
 """
+from sqlfluff.core.dialects import load_raw_dialect
 from sqlfluff.core.parser import (
-    OneOf,
     AnyNumberOf,
     AnySetOf,
     Anything,
-    Ref,
-    Sequence,
-    Bracketed,
     BaseSegment,
+    Bracketed,
+    CodeSegment,
     Delimited,
     Matchable,
     Nothing,
+    OneOf,
     OptionallyBracketed,
+    Ref,
+    RegexLexer,
+    RegexParser,
+    SegmentGenerator,
+    Sequence,
 )
-from sqlfluff.core.dialects import load_raw_dialect
+from sqlfluff.dialects import dialect_ansi as ansi
+from sqlfluff.dialects import dialect_postgres as postgres
 from sqlfluff.dialects.dialect_redshift_keywords import (
     redshift_reserved_keywords,
     redshift_unreserved_keywords,
 )
-from sqlfluff.dialects import dialect_postgres as postgres
-from sqlfluff.dialects import dialect_ansi as ansi
 
 postgres_dialect = load_raw_dialect("postgres")
 ansi_dialect = load_raw_dialect("ansi")
@@ -160,6 +164,18 @@ redshift_dialect.replace(
         ),
         Ref("AliasExpressionSegment", optional=True),
     ),
+    NakedIdentifierSegment=SegmentGenerator(
+        lambda dialect: RegexParser(
+            # Optionally begins with # for temporary tables. Otherwise
+            # must only contain digits, letters, underscore, and $ but
+            # can’t be all digits.
+            r"#?([A-Z_]+|[0-9]+[A-Z_$])[A-Z0-9_$]*",
+            CodeSegment,
+            name="naked_identifier",
+            type="identifier",
+            anti_template=r"^(" + r"|".join(dialect.sets("reserved_keywords")) + r")$",
+        )
+    ),
     ColumnReferenceSegment=Delimited(
         Sequence(
             ansi.ColumnReferenceSegment,
@@ -183,6 +199,13 @@ redshift_dialect.replace(
     ),
 )
 
+redshift_dialect.patch_lexer_matchers(
+    [
+        # add optional leading # to code for temporary tables
+        RegexLexer("code", r"#?[0-9a-zA-Z_]+[0-9a-zA-Z_$]*", CodeSegment),
+    ]
+)
+
 
 # Inherit from the Postgres ObjectReferenceSegment this way so we can inherit
 # other segment types from it.
@@ -655,6 +678,7 @@ class AlterTableActionSegment(BaseSegment):
             Ref.keyword("COLUMN", optional=True),
             Ref("ColumnReferenceSegment"),
             Ref("DatatypeSegment"),
+            Sequence("DEFAULT", Ref("ExpressionSegment"), optional=True),
             Sequence("COLLATE", Ref("QuotedLiteralSegment"), optional=True),
             AnyNumberOf(Ref("ColumnConstraintSegment")),
         ),
@@ -1596,6 +1620,21 @@ class DropProcedureStatementSegment(BaseSegment):
     )
 
 
+class AlterDefaultPrivilegesSchemaObjectsSegment(
+    postgres.AlterDefaultPrivilegesSchemaObjectsSegment
+):
+    """`ALTER DEFAULT PRIVILEGES` schema object types.
+
+    https://docs.aws.amazon.com/redshift/latest/dg/r_ALTER_DEFAULT_PRIVILEGES.html
+    """
+
+    match_grammar = (
+        postgres.AlterDefaultPrivilegesSchemaObjectsSegment.match_grammar.copy(
+            insert=[Sequence("PROCEDURES")]
+        )
+    )
+
+
 class DeclareStatementSegment(BaseSegment):
     """A `DECLARE` statement.
 
@@ -1889,6 +1928,7 @@ class StatementSegment(postgres.StatementSegment):
             Ref("AnalyzeCompressionStatementSegment"),
             Ref("VacuumStatementSegment"),
             Ref("AlterProcedureStatementSegment"),
+            Ref("CallStatementSegment"),
         ],
     )
 
@@ -2228,3 +2268,24 @@ class ObjectUnpivotSegment(BaseSegment):
         "AT",
         Ref("SingleIdentifierGrammar"),
     )
+
+
+class CallStatementSegment(BaseSegment):
+    """A `CALL` statement.
+
+    https://docs.aws.amazon.com/redshift/latest/dg/r_CALL_procedure.html
+    """
+
+    type = "call_statement"
+    match_grammar = Sequence(
+        "CALL",
+        Ref("FunctionSegment"),
+    )
+
+
+class SelectClauseModifierSegment(postgres.SelectClauseModifierSegment):
+    """Things that come after SELECT but before the columns."""
+
+    match_grammar = postgres.SelectClauseModifierSegment.match_grammar.copy(
+        insert=[Sequence("TOP", Ref("NumericLiteralSegment"))],
+    )
