@@ -17,10 +17,15 @@ ansi_dialect = load_raw_dialect("ansi")
 
 sqlite_dialect = ansi_dialect.copy_as("sqlite")
 
+sqlite_dialect.sets("reserved_keywords").update(["AUTOINCREMENT"])
+
 sqlite_dialect.replace(
     BooleanBinaryOperatorGrammar=OneOf(
         Ref("AndOperatorGrammar"), Ref("OrOperatorGrammar"), "REGEXP"
     ),
+    PrimaryKeyGrammar=Sequence(
+        "PRIMARY", "KEY", Sequence("AUTOINCREMENT", optional=True)
+    ),
 )
 
 
