@@ -6,7 +6,6 @@ from sqlfluff.core.parser import (
     IdentitySet,
     NewlineSegment,
     PositionMarker,
-    WhitespaceSegment,
 )
 
 from sqlfluff.core.rules.base import BaseRule, LintFix, LintResult, RuleContext
@@ -22,25 +21,23 @@ from sqlfluff.core.rules.functional import sp
 @document_fix_compatible
 @document_configuration
 class Rule_L018(BaseRule):
-    """``WITH`` clause closing bracket should be aligned with ``WITH`` keyword.
+    """``WITH`` clause closing bracket should be on a new line.
 
     **Anti-pattern**
 
-    The ``•`` character represents a space.
-    In this example, the closing bracket is not aligned with ``WITH`` keyword.
+    In this example, the closing bracket is on the same line as CTE.
 
     .. code-block:: sql
        :force:
 
         WITH zoo AS (
-            SELECT a FROM foo
-        ••••)
+            SELECT a FROM foo)
 
         SELECT * FROM zoo
 
     **Best practice**
 
-    Remove the spaces to align the ``WITH`` keyword with the closing bracket.
+    Move the closing bracket on a new line.
 
     .. code-block:: sql
 
@@ -55,7 +52,6 @@ class Rule_L018(BaseRule):
     groups = ("all", "core")
     _works_on_unparsable = False
     needs_raw_stack = True
-    config_keywords = ["tab_space_size"]
 
     def _eval(self, context: RuleContext):
         """WITH clause closing bracket should be aligned with WITH keyword.
@@ -79,26 +75,6 @@ class Rule_L018(BaseRule):
                 # we should raise that.
                 raise RuntimeError("Didn't find WITH keyword!")
 
-            def indent_size_up_to(segs):
-                seg_buff = []
-                # Get any segments running up to the WITH
-                for elem in reversed(segs):
-                    if elem.is_type("newline"):
-                        break
-                    elif elem.is_meta:
-                        continue
-                    else:
-                        seg_buff.append(elem)
-                # reverse the indent if we have one
-                if seg_buff:
-                    seg_buff = list(reversed(seg_buff))
-                indent_str = "".join(seg.raw for seg in seg_buff).replace(
-                    "\t", " " * self.tab_space_size
-                )
-                indent_size = len(indent_str)
-                return indent_size, indent_str
-
-            with_indent, with_indent_str = indent_size_up_to(raw_stack_buff)
             # Find the end brackets for the CTE *query* (i.e. ignore optional
             # list of CTE columns).
             cte_end_brackets = IdentitySet()
@@ -121,51 +97,40 @@ class Rule_L018(BaseRule):
                         raw_stack_buff.append(seg)
                     continue
 
-                closing_bracket_indent, _ = indent_size_up_to(raw_stack_buff)
-                indent_diff = closing_bracket_indent - with_indent
-                # Is indent of closing bracket not the same as
-                # indent of WITH keyword.
                 if seg.pos_marker.line_no == seg_line_no:
                     # Skip if it's the one-line version. That's ok
-                    pass
-                elif indent_diff < 0:
+                    continue
+
+                # Is it all whitespace before the bracket on this line?
+                assert seg.pos_marker
+
+                contains_non_whitespace = False
+                for elem in context.segment.raw_segments:
+                    if (
+                        cast(PositionMarker, elem.pos_marker).line_no
+                        == seg.pos_marker.line_no
+                        and cast(PositionMarker, elem.pos_marker).line_pos
+                        <= seg.pos_marker.line_pos
+                    ):
+                        if elem is seg:
+                            break
+                        elif elem.is_type("newline"):
+                            contains_non_whitespace = False
+                        elif not elem.is_type("dedent") and not elem.is_type(
+                            "whitespace"
+                        ):
+                            contains_non_whitespace = True
+
+                if contains_non_whitespace:
+                    # We have to move it to a newline
                     return LintResult(
                         anchor=seg,
                         fixes=[
                             LintFix.create_before(
                                 seg,
-                                [WhitespaceSegment(" " * (-indent_diff))],
+                                [
+                                    NewlineSegment(),
+                                ],
                             )
                         ],
                     )
-                elif indent_diff > 0:
-                    # Is it all whitespace before the bracket on this line?
-                    assert seg.pos_marker
-                    prev_segs_on_line = [
-                        elem
-                        for elem in context.segment.raw_segments
-                        if cast(PositionMarker, elem.pos_marker).line_no
-                        == seg.pos_marker.line_no
-                        and cast(PositionMarker, elem.pos_marker).line_pos
-                        < seg.pos_marker.line_pos
-                    ]
-                    if all(elem.is_type("whitespace") for elem in prev_segs_on_line):
-                        # We can move it back, it's all whitespace
-                        fixes = [
-                            LintFix.create_before(
-                                seg,
-                                [WhitespaceSegment(with_indent_str)],
-                            )
-                        ] + [LintFix.delete(elem) for elem in prev_segs_on_line]
-                    else:
-                        # We have to move it to a newline
-                        fixes = [
-                            LintFix.create_before(
-                                seg,
-                                [
-                                    NewlineSegment(),
-                                    WhitespaceSegment(with_indent_str),
-                                ],
-                            )
-                        ]
-                    return LintResult(anchor=seg, fixes=fixes)
