@@ -7,7 +7,11 @@ from sqlfluff.core.parser import WhitespaceSegment
 from sqlfluff.core.parser import BaseSegment, NewlineSegment
 from sqlfluff.core.parser.segments.base import IdentitySet
 from sqlfluff.core.rules.base import BaseRule, LintFix, LintResult, RuleContext
-from sqlfluff.core.rules.doc_decorators import document_fix_compatible, document_groups
+from sqlfluff.core.rules.doc_decorators import (
+    document_configuration,
+    document_fix_compatible,
+    document_groups,
+)
 from sqlfluff.core.rules.functional import Segments
 import sqlfluff.core.rules.functional.segment_predicates as sp
 
@@ -26,13 +30,15 @@ class SelectTargetsInfo(NamedTuple):
 
 
 @document_groups
+@document_configuration
 @document_fix_compatible
 class Rule_L036(BaseRule):
     """Select targets should be on a new line unless there is only one select target.
 
     .. note::
-       A wildcard is (``SELECT *``) is not considered a single select target so
-       always requires a new line.
+       By default, a wildcard (e.g. ``SELECT *``) is considered a single select target.
+       If you want it to be treated as multiple select targets, configure
+       ``wildcard_policy = multiple``.
 
     **Anti-pattern**
 
@@ -72,16 +78,25 @@ class Rule_L036(BaseRule):
     """
 
     groups = ("all",)
+    config_keywords = ["wildcard_policy"]
 
     def _eval(self, context: RuleContext):
+        self.wildcard_policy: str
         if context.segment.is_type("select_clause"):
             select_targets_info = self._get_indexes(context)
-            if len(select_targets_info.select_targets) == 1:
+            select_clause = context.functional.segment
+            wildcards = select_clause.children(
+                sp.is_type("select_clause_element")
+            ).children(sp.is_type("wildcard_expression"))
+            has_wildcard = bool(wildcards)
+            if len(select_targets_info.select_targets) == 1 and (
+                not has_wildcard or self.wildcard_policy == "single"
+            ):
                 return self._eval_single_select_target_element(
                     select_targets_info,
                     context,
                 )
-            elif len(select_targets_info.select_targets) > 1:
+            elif len(select_targets_info.select_targets):
                 return self._eval_multiple_select_target_elements(
                     select_targets_info, context.segment
                 )
@@ -195,18 +210,12 @@ class Rule_L036(BaseRule):
     ):
         select_clause = context.functional.segment
         parent_stack = context.parent_stack
-        wildcards = select_clause.children(
-            sp.is_type("select_clause_element")
-        ).children(sp.is_type("wildcard_expression"))
-        is_wildcard = bool(wildcards)
-        if is_wildcard:
-            wildcard_select_clause_element = wildcards[0]
 
         if (
             select_targets_info.select_idx
             < select_targets_info.first_new_line_idx
             < select_targets_info.first_select_target_idx
-        ) and (not is_wildcard):
+        ):
             # Do we have a modifier?
             select_children = select_clause.children()
             modifier: Optional[Segments]
@@ -416,33 +425,4 @@ class Rule_L036(BaseRule):
                 anchor=select_clause.get(),
                 fixes=fixes,
             )
-
-        # If we have a wildcard on the same line as the FROM keyword, but not the same
-        # line as the SELECT keyword, we need to move the FROM keyword to its own line.
-        # i.e.
-        # SELECT
-        #   * FROM foo
-        if select_targets_info.from_segment:
-            if (
-                is_wildcard
-                and (
-                    select_clause[0].pos_marker.working_line_no
-                    != select_targets_info.from_segment.pos_marker.working_line_no
-                )
-                and (
-                    wildcard_select_clause_element.pos_marker.working_line_no
-                    == select_targets_info.from_segment.pos_marker.working_line_no
-                )
-            ):
-                fixes = [
-                    LintFix.delete(ws) for ws in select_targets_info.pre_from_whitespace
-                ]
-                fixes.append(
-                    LintFix.create_before(
-                        select_targets_info.from_segment,
-                        [NewlineSegment()],
-                    )
-                )
-                return LintResult(anchor=select_clause.get(), fixes=fixes)
-
         return None
